@@ -1,22 +1,16 @@
 use smallvec::SmallVec;
-use std::mem::take;
-use std::{collections::HashMap, mem::replace};
+use std::mem::{replace, take};
 use swc_atoms::js_word;
-use swc_common::{util::map::Map, Mark, Spanned, SyntaxContext, DUMMY_SP};
+use swc_common::{collections::AHashMap, util::map::Map, Mark, Spanned, SyntaxContext, DUMMY_SP};
 use swc_ecma_ast::*;
 use swc_ecma_transforms_base::helper;
-use swc_ecma_utils::private_ident;
-use swc_ecma_utils::quote_ident;
-use swc_ecma_utils::quote_str;
-use swc_ecma_utils::undefined;
 use swc_ecma_utils::{
-    contains_arguments, contains_this_expr, find_ids, ident::IdentLike, prepend, var::VarCollector,
-    ExprFactory, Id, StmtLike,
+    contains_arguments, contains_this_expr, find_ids, ident::IdentLike, prepend, private_ident,
+    quote_ident, quote_str, undefined, var::VarCollector, ExprFactory, Id, StmtLike,
 };
-use swc_ecma_visit::noop_visit_type;
 use swc_ecma_visit::{
-    noop_fold_type, noop_visit_mut_type, Fold, FoldWith, Node, Visit, VisitMut, VisitMutWith,
-    VisitWith,
+    noop_fold_type, noop_visit_mut_type, noop_visit_type, Fold, FoldWith, Node, Visit, VisitMut,
+    VisitMutWith, VisitWith,
 };
 
 ///
@@ -51,7 +45,7 @@ enum ScopeKind {
         /// Produced by identifier reference and consumed by for-of/in loop.
         used: Vec<Id>,
         /// Map of original identifer to modified syntax context
-        mutated: HashMap<Id, SyntaxContext>,
+        mutated: AHashMap<Id, SyntaxContext>,
     },
     Fn,
     Block,
@@ -70,6 +64,8 @@ impl BlockScoping {
     where
         T: FoldWith<Self>,
     {
+        let len = self.scope.len();
+
         let remove = match kind {
             ScopeKind::ForLetLoop { .. } => false,
             _ => true,
@@ -78,7 +74,7 @@ impl BlockScoping {
         let node = node.fold_with(self);
 
         if remove {
-            self.scope.pop();
+            self.scope.truncate(len);
         }
 
         node
@@ -433,6 +429,14 @@ impl Fold for BlockScoping {
         }
     }
 
+    fn fold_block_stmt(&mut self, n: BlockStmt) -> BlockStmt {
+        let vars = take(&mut self.vars);
+        let n = n.fold_children_with(self);
+        debug_assert_eq!(self.vars, vec![]);
+        self.vars = vars;
+        n
+    }
+
     fn fold_constructor(&mut self, f: Constructor) -> Constructor {
         Constructor {
             key: f.key.fold_with(self),
@@ -567,6 +571,10 @@ impl Fold for BlockScoping {
         node
     }
 
+    fn fold_module_items(&mut self, n: Vec<ModuleItem>) -> Vec<ModuleItem> {
+        self.fold_stmt_like(n)
+    }
+
     fn fold_setter_prop(&mut self, f: SetterProp) -> SetterProp {
         SetterProp {
             key: f.key.fold_with(self),
@@ -576,6 +584,10 @@ impl Fold for BlockScoping {
         }
     }
 
+    fn fold_stmts(&mut self, n: Vec<Stmt>) -> Vec<Stmt> {
+        self.fold_stmt_like(n)
+    }
+
     fn fold_var_decl(&mut self, var: VarDecl) -> VarDecl {
         let old = self.var_decl_kind;
         self.var_decl_kind = var.kind;
@@ -612,22 +624,6 @@ impl Fold for BlockScoping {
 
         WhileStmt { body, test, ..node }
     }
-
-    fn fold_block_stmt(&mut self, n: BlockStmt) -> BlockStmt {
-        let vars = take(&mut self.vars);
-        let n = n.fold_children_with(self);
-        debug_assert_eq!(self.vars, vec![]);
-        self.vars = vars;
-        n
-    }
-
-    fn fold_module_items(&mut self, n: Vec<ModuleItem>) -> Vec<ModuleItem> {
-        self.fold_stmt_like(n)
-    }
-
-    fn fold_stmts(&mut self, n: Vec<Stmt>) -> Vec<Stmt> {
-        self.fold_stmt_like(n)
-    }
 }
 
 impl BlockScoping {
@@ -750,7 +746,7 @@ struct FlowHelper<'a> {
     has_break: bool,
     has_return: bool,
     all: &'a Vec<Id>,
-    mutated: HashMap<Id, SyntaxContext>,
+    mutated: AHashMap<Id, SyntaxContext>,
     in_switch_case: bool,
 }
 
@@ -879,7 +875,7 @@ impl Fold for FlowHelper<'_> {
 }
 
 struct MutationHandler<'a> {
-    map: &'a mut HashMap<Id, SyntaxContext>,
+    map: &'a mut AHashMap<Id, SyntaxContext>,
     in_function: bool,
     this: Option<Ident>,
     arguments: Option<Ident>,
@@ -994,6 +990,10 @@ struct FunctionFinder {
 impl Visit for FunctionFinder {
     noop_visit_type!();
 
+    fn visit_arrow_expr(&mut self, _: &ArrowExpr, _: &dyn Node) {
+        self.found = true;
+    }
+
     fn visit_function(&mut self, _: &Function, _: &dyn Node) {
         self.found = true
     }
