@@ -1,14 +1,13 @@
 use fxhash::FxHashSet;
 use std::{iter, mem};
 use swc_atoms::JsWord;
-use swc_common::SyntaxContext;
-use swc_common::{Mark, Spanned, DUMMY_SP};
+use swc_common::{Mark, Spanned, SyntaxContext, DUMMY_SP};
 use swc_ecma_ast::*;
-use swc_ecma_transforms_base::ext::AsOptExpr;
-use swc_ecma_transforms_base::ext::PatOrExprExt;
-use swc_ecma_transforms_base::helper;
-use swc_ecma_utils::quote_ident;
-use swc_ecma_utils::{alias_ident_for, alias_if_required, prepend, ExprFactory};
+use swc_ecma_transforms_base::{
+    ext::{AsOptExpr, PatOrExprExt},
+    helper,
+};
+use swc_ecma_utils::{alias_ident_for, alias_if_required, prepend, quote_ident, ExprFactory};
 use swc_ecma_visit::{noop_fold_type, Fold, FoldWith};
 
 pub(super) struct FieldAccessFolder<'a> {
